/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::IndexMemory
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "IndexMemory.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>
#include <ArmarXCore/core/ice_conversions/ice_conversions_templates.h>

#include <RobotAPI/libraries/armem/server/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem_index/aron/Named.aron.generated.h>
#include <RobotAPI/libraries/armem_index/aron/Spatial.aron.generated.h>
#include <RobotAPI/libraries/armem_index/memory_ids.h>


namespace armarx
{

    armarx::PropertyDefinitionsPtr
    IndexMemory::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs =
            new ComponentPropertyDefinitions(getConfigIdentifier());

        setMemoryName(armem::index::memoryID.memoryName);

        defs->optional(properties.maxHistorySize, "p.maxHistorySize", "The maximum size of entity histories.")
                .setMin(1);

        return defs;
    }


    std::string
    IndexMemory::getDefaultName() const
    {
        return "IndexMemory";
    }


    void
    IndexMemory::onInitComponent()
    {
        size_t maxHistorySize = static_cast<size_t>(std::max(1, properties.maxHistorySize));
        workingMemory()
            .addCoreSegment(armem::index::namedSegmentID.coreSegmentName,
                            armem::index::arondto::Named::ToAronType())
            .setMaxHistorySize(maxHistorySize);

        workingMemory()
            .addCoreSegment(armem::index::spatialSegmentID.coreSegmentName,
                            armem::index::arondto::Spatial::ToAronType())
            .setMaxHistorySize(maxHistorySize);
    }


    void
    IndexMemory::onConnectComponent()
    {
        {
            createRemoteGuiTab();
            RemoteGui_startRunningTask();
        }
    }


    void
    IndexMemory::onDisconnectComponent()
    {
    }


    void
    IndexMemory::onExitComponent()
    {
    }


    void
    IndexMemory::createRemoteGuiTab()
    {
        using namespace armarx::RemoteGui::Client;

        {
        }

        VBoxLayout root = {VSpacer()};
        RemoteGui_createTab(getName(), root, &tab);
    }


    void
    IndexMemory::RemoteGui_update()
    {
    }

} // namespace armarx
