#include "AffordanceLoader.h"

#include <ArmarXCore/core/eigen/ice_conversions.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::priorknowledge::util
{
    std::vector<StaticAffordance>
    AffordanceLoader::LoadStaticAffordances(const std::string& sourceId, const nlohmann::json& js)
    {
        std::vector<StaticAffordance> ret;
        if (not js.contains("static_affordances"))
        {
            ARMARX_WARNING << "The affordances file has the wrong structure. Missing key "
                              "'static_affordances'.";
            return ret;
        }

        for (const auto& affordance : js["static_affordances"].get<std::vector<std::string>>())
        {
            StaticAffordance a(sourceId, affordance);
            ret.push_back(a);
        }
        return ret;
    }

    std::vector<LocatableAffordance>
    AffordanceLoader::LoadLocatableAffordances(const std::string& sourceId,
                                               const nlohmann::json& js)
    {
        std::vector<LocatableAffordance> ret;
        if (not js.contains("locatable_affordances"))
        {
            ARMARX_WARNING << "The affordances file has the wrong structure. Missing key "
                              "'locatable_affordances'.";
            return ret;
        }

        for (const auto& [affordance, locationIds] :
             js["locatable_affordances"].get<std::map<std::string, std::vector<std::string>>>())
        {
            for (const auto& locationId : locationIds)
            {
                LocatableAffordance a(sourceId, affordance, locationId);
                ret.push_back(a);
            }
        }
        return ret;
    }
} // namespace armarx::priorknowledge::util
