#include "MemoryItem.h"

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/time/TimeUtil.h>

#include <RobotAPI/libraries/armem/server/wm/memory_definitions.h>

namespace armarx::armem::server::ltm::detail
{
    MemoryItem::MemoryItem(const std::string& exportName, const MemoryID& id) :
        processors(std::make_shared<Processors>()), exportName(exportName), _id(id)
    {
    }

    MemoryItem::MemoryItem(const std::string& exportName,
                           const MemoryID& id,
                           const std::shared_ptr<Processors>& p) :
        processors(p), exportName(exportName), _id(id)
    {
    }

    void
    MemoryItem::setExportName(const std::string& id)
    {
        exportName = id;
        _setExportName(id);
    }

    void
    MemoryItem::setMemoryID(const MemoryID& id)
    {
        _id = id;
        _setMemoryID(id);
    }

    MemoryID
    MemoryItem::id() const
    {
        return _id;
    }

    std::string
    MemoryItem::name() const
    {
        return _id.getLeafItem();
    }

    void
    MemoryItem::setMemoryName(const std::string& memoryName)
    {
        _id.memoryName = memoryName;
    }
} // namespace armarx::armem::server::ltm::detail
